#
# a font selector (based on code of Keith Vetter from wiki.tcl.tk)
#
# Copyright (C) 2008 Falk Richter
#
# ::FR_fontSelect::Create
#
# -parent "<window which starts FR_fontSelect> | . (default)"
# -title "<your window title> | "Font selector" (default)"
# -initialfont "<start with font> | nothing (default)"

package provide FR_fontSelect 0.1

package require Tk
package require FR_general


namespace eval ::FR_fontSelect \
{
 namespace export Create
}


proc ::FR_fontSelect::ButtonAction {Path ReturnValue} \
{
 variable VAR
  
 if { $ReturnValue } \
 {
  set VAR($Path,Return) $VAR($Path,NewFont)
 } else \
 {
  set VAR($Path,Return) ""
 }
 
 focus $VAR($Path,Parent)
 destroy .$Path
 set VAR($Path,Stop) 1
}


proc ::FR_fontSelect::Create {args} \
{
 variable VAR
 
 # first we generate a toplevel name who doesn't exist yet
 set i 1
 while {[winfo exists .fontsel_window$i]} {incr i}
 set Path "fontsel_window$i"
 toplevel .$Path
 
 set VAR($Path,Parent) .
 set VAR($Path,Title) [msgcat::mc "Font selector"] 
 set VAR($Path,Stop) 0
 set VAR($Path,Return) ""
 set VAR($Path,IniFont) ""
 
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
  set Arg [lindex $args $i]
  switch -glob -- $Arg \
  {
     -parent       {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
     -title        {set VAR($Path,Title) [lindex $args [expr {$i+1}]]} \
     -initialfont  {set VAR($Path,IniFont) [lindex $args [expr {$i+1}]]} \
     --            {break} \
     *             {puts "Unknown option: $Arg"; exit 1}
  }
 }
 
 ::FR_fontSelect::GUI $Path
 
 ::FR_fontSelect::FirstStart $Path
 
 ::FR_general::CenterWindow .$Path $VAR($Path,Parent)
 
 wm deiconify .$Path
 
 wm geometry .$Path 400x400
 
 wm protocol .$Path WM_DELETE_WINDOW [list ::FR_fontSelect::ButtonAction $Path 0]
 
 vwait ::FR_fontSelect::VAR($Path,Stop)
 return $VAR($Path,Return) 
}
 
  
#---we create all widgets
proc ::FR_fontSelect::GUI {Path} \
{
 variable VAR 

 wm title .$Path $VAR($Path,Title)
 wm withdraw .$Path

 labelframe .$Path.frame1 -relief groove -text [msgcat::mc "Font type"] -foreground blue
 labelframe .$Path.frame2 -relief groove -text [msgcat::mc "Font options"] -foreground blue
 labelframe .$Path.frame3 -relief groove -text [msgcat::mc "Preview"] -foreground blue
 frame .$Path.frame4 -relief flat

 pack  .$Path.frame1 -side top -fill x -padx 5 -pady 5
 pack  .$Path.frame2 -side top -fill x -padx 5 -pady 5
 pack  .$Path.frame4 -side bottom -fill x -padx 5 -pady 5
 pack  .$Path.frame3 -side top -expand yes -fill both -padx 5 -pady 5
 

 # We set a variable for each frame - so we can change the order of the frames later easier 
 set VAR($Path,mainframe) "$Path.frame1"
 set VAR($Path,optionsframe) "$Path.frame2"
 set VAR($Path,previewframe) "$Path.frame3"
 set VAR($Path,bottomframe) "$Path.frame4"
 
 # the font type frame
 label .$VAR($Path,mainframe).family_label -text [msgcat::mc "Family"]
 listbox .$VAR($Path,mainframe).family -width 25 -background white -activestyle none -selectmode single -height 7 -yscrollcommand [list .$VAR($Path,mainframe).family_sb set]
 scrollbar .$VAR($Path,mainframe).family_sb -orient vertical -command [list .$VAR($Path,mainframe).family yview]

 label .$VAR($Path,mainframe).style_label -text [msgcat::mc "Style"]
 listbox .$VAR($Path,mainframe).style -width 10 -background white -activestyle none -selectmode single -height 7 -yscrollcommand [list .$VAR($Path,mainframe).style_sb set]
 scrollbar .$VAR($Path,mainframe).style_sb -orient vertical -command [list .$VAR($Path,mainframe).style yview]

 label .$VAR($Path,mainframe).size_label -text [msgcat::mc "Size"]
 listbox .$VAR($Path,mainframe).size -width 3 -background white -activestyle none -selectmode single -height 7 -yscrollcommand [list .$VAR($Path,mainframe).size_sb set]
 scrollbar .$VAR($Path,mainframe).size_sb -orient vertical -command [list .$VAR($Path,mainframe).size yview]
 
 grid .$VAR($Path,mainframe).family_label -row 0 -column 0 -sticky w -padx {5 0} -pady {5 0}
 grid .$VAR($Path,mainframe).family -row 1 -column 0 -sticky ew -padx {5 0} -pady {0 5}
 grid .$VAR($Path,mainframe).family_sb -row 1 -column 1 -sticky ns -padx {0 5} -pady {0 5}
 
 grid .$VAR($Path,mainframe).style_label -row 0 -column 2 -sticky w -padx {5 0} -pady {5 0}
 grid .$VAR($Path,mainframe).style -row 1 -column 2 -sticky ew -padx {5 0} -pady {0 5}
 grid .$VAR($Path,mainframe).style_sb -row 1 -column 3 -sticky ns -padx {0 5} -pady {0 5}

 grid .$VAR($Path,mainframe).size_label -row 0 -column 4 -sticky w -padx {5 0} -pady {5 0}
 grid .$VAR($Path,mainframe).size -row 1 -column 4 -sticky ew -padx {5 0} -pady {0 5}
 grid .$VAR($Path,mainframe).size_sb -row 1 -column 5 -sticky ns -padx {0 5} -pady {0 5}

 grid columnconfigure .$VAR($Path,mainframe) 0 -weight 1
 grid columnconfigure .$VAR($Path,mainframe) 2 -weight 1
 grid columnconfigure .$VAR($Path,mainframe) 4 -weight 1
 
 bind .$VAR($Path,mainframe).family <<ListboxSelect>> [list ::FR_fontSelect::Click $Path family]
 bind .$VAR($Path,mainframe).style <<ListboxSelect>> [list ::FR_fontSelect::Click $Path style]
 bind .$VAR($Path,mainframe).size <<ListboxSelect>> [list ::FR_fontSelect::Click $Path size]
  
 # the font options frame
 checkbutton .$VAR($Path,optionsframe).italic -variable ::FR_fontSelect::VAR($Path,italic) -text [msgcat::mc "Italic"] -command [list ::FR_fontSelect::Click $Path italic]
 checkbutton .$VAR($Path,optionsframe).overstrike -variable ::FR_fontSelect::VAR($Path,overstrike) -text [msgcat::mc "Overstrike"] -command [list ::FR_fontSelect::Click $Path overstrike]
 checkbutton .$VAR($Path,optionsframe).underline -variable ::FR_fontSelect::VAR($Path,underline) -text [msgcat::mc "Underline"] -command [list ::FR_fontSelect::Click $Path underline]

 grid .$VAR($Path,optionsframe).italic -row 0 -column 0 -sticky w -padx 5 -pady 5
 grid .$VAR($Path,optionsframe).overstrike -row 0 -column 1 -sticky w -padx 5 -pady 5
 grid .$VAR($Path,optionsframe).underline -row 0 -column 2 -sticky w -padx 5 -pady 5
 grid columnconfigure .$VAR($Path,optionsframe) 3 -weight 1
 
 # the preview frame
 label .$VAR($Path,previewframe).preview -text "AaByZz" -background white
 
 grid  .$VAR($Path,previewframe).preview -row 0 -column 0 -sticky nsew -padx 5 -pady 5
 grid columnconfigure .$VAR($Path,previewframe) 0 -weight 1
 grid rowconfigure .$VAR($Path,previewframe) 0 -weight 1
  
 # the bottom frame
 set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
  
 button .$VAR($Path,bottomframe).cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::FR_fontSelect::ButtonAction $Path 0]
 button .$VAR($Path,bottomframe).okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::FR_fontSelect::ButtonAction $Path 1]
   
 pack .$VAR($Path,bottomframe).okay -side left -padx {5 0} -pady 5 -expand yes -fill x
 pack .$VAR($Path,bottomframe).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
 bind .$Path <KeyPress-Escape> [list .$VAR($Path,bottomframe).cancel invoke]
 bind .$Path <KeyPress-Return> [list .$VAR($Path,bottomframe).okay invoke]
}
         

proc ::FR_fontSelect::PreView {Path} \
{
 variable VAR
 
 set VAR($Path,NewFont) [list $VAR($Path,Family) $VAR($Path,Size)]
 
 if {$VAR($Path,italic)} {lappend VAR($Path,NewFont) "italic"}
 if {$VAR($Path,roman)} {lappend VAR($Path,NewFont) "roman"}
 if {$VAR($Path,normal)} {lappend VAR($Path,NewFont) "normal"}
 if {$VAR($Path,bold)} {lappend VAR($Path,NewFont) "bold"}
 if {$VAR($Path,overstrike)} {lappend VAR($Path,NewFont) "overstrike"}
 if {$VAR($Path,underline)} {lappend VAR($Path,NewFont) "underline"} 

 .$VAR($Path,previewframe).preview configure -font $VAR($Path,NewFont)

 update idletasks
 
 return 
}



proc ::FR_fontSelect::Click {Path Item} \
{
 variable VAR
 
 set Style ""
 
 switch -- $Item \
 {
  family        {set VAR($Path,Family) [.$VAR($Path,mainframe).family get [.$VAR($Path,mainframe).family curselection]] } \
  style         {set Style [.$VAR($Path,mainframe).style get [.$VAR($Path,mainframe).style curselection]]} \
  size          {set VAR($Path,Size) [.$VAR($Path,mainframe).size get [.$VAR($Path,mainframe).size curselection]]} \
  italic        {} \
  overstrike    {} \
  underline     {} \
  *             {puts "Unknown option: $Item"}
 }
 
   
 if {$Style eq [msgcat::mc "normal"]} \
 {
  set VAR($Path,normal) 1
  set VAR($Path,bold) 0
  
 } elseif {$Style eq [msgcat::mc "bold"]} \
 {
  set VAR($Path,normal) 0
  set VAR($Path,bold) 1
 }
 
 ::FR_fontSelect::PreView $Path
}


# we parse the initial font and set / fill out all widgets
proc ::FR_fontSelect::FirstStart {Path} \
{
 variable VAR
 
 set FontFamilies [lsort -dictionary [font families]]
 set Style [list [msgcat::mc "normal"] [msgcat::mc "bold"]]
 set Size [list 8 9 10 11 12 14 16 18 20 22 24 26 28 36 48 72]

 #.$VAR($Path,mainframe).family delete 0 end
 #.$VAR($Path,mainframe).style delete 0 end
 #.$VAR($Path,mainframe).size delete 0 end
 
 foreach Arg $FontFamilies {.$VAR($Path,mainframe).family insert end $Arg}
 foreach Arg $Style {.$VAR($Path,mainframe).style insert end $Arg}
 foreach Arg $Size {.$VAR($Path,mainframe).size insert end $Arg}
 
 # we check what is the default font of your system
 if {$VAR($Path,IniFont) eq ""} \
 {
  set VAR($Path,IniFont) [.$VAR($Path,previewframe).preview cget -font]
 }
 
 # we preset the new font with the old one 
 # (in case nothing is chosen but the ok button is pressed)
 set VAR($Path,NewFont) $VAR($Path,IniFont)
 
 # now we show the initial font family
 set IniFontFamilies [lindex $VAR($Path,IniFont) 0]
 
 # in tcl8.5 font families returns all values in lower case characters but
 # tcl8.4 doesn't not. We are using string match because lsearch -nocase exists
 # only in tcl8.5
 set SearchResult "-1"
 
 for {set i 0} {$i < [llength $FontFamilies]} {incr i} \
 {
   if {[string match -nocase $IniFontFamilies [lindex $FontFamilies $i]]} \
   {
     set SearchResult $i
     break
   }
 }
 
 # we show the search result or the first element
 if { $SearchResult == -1} {set SearchResult 0}
 .$VAR($Path,mainframe).family see $SearchResult
 
 set VAR($Path,Family) [lindex $FontFamilies $SearchResult]
 
 # now we show the inital font size
 set IniSize [lindex $VAR($Path,IniFont) 1]
 
 set SearchResult [lsearch -exact $Size [string trimleft $IniSize "-"]]
 
 # we show the search result or the first element    
 if { $SearchResult == -1} {set SearchResult 0}
 .$VAR($Path,mainframe).size see $SearchResult
 
 set VAR($Path,Size) [lindex $Size $SearchResult]
 
 # we parse the optional arguments of the initial font
 set VAR($Path,italic) 0
 set VAR($Path,roman) 1
 set VAR($Path,normal) 1
 set VAR($Path,bold) 0
 set VAR($Path,overstrike) 0
 set VAR($Path,underline) 0
 
 set IniFontLength [llength $VAR($Path,IniFont)]
 
 if {$IniFontLength > 2} \
 {
  for {set i 2} {$i < $IniFontLength} {incr i} \
  {
   set Arg [lindex $VAR($Path,IniFont) $i]
   switch -glob -- $Arg \
   {
     italic        {set VAR($Path,italic) 1; set VAR($Path,roman) 0} \
     roman         {} \
     normal        {} \
     bold          {set VAR($Path,bold) 1; set VAR($Path,normal) 0} \
     overstrike    {set VAR($Path,overstrike) 1} \
     underline     {set VAR($Path,underline) 1} \
     *             {puts "Unknown option: $Arg"}
   }
  }
 }
 
 ::FR_fontSelect::PreView $Path
}           
                      
#::FR_fontSelect::Create -initialfont [list Arial 12 bold italic underline]