package provide FR_general 0.2

namespace eval ::FR_general \
{
 namespace export labelfieldbutton labelfield label_listbox_scrollbar headline Topic Line output \
 TextwidgetWithScrollbars PadFrame PackHelp_1 PackHelp_2 CenterWindow window_maximise \
 ButtonsWithSameWidth ScrollWindow ScrollWindowShowWidget ScrollWindowIncrement ScrollWindowRefresh MenuBinding \
 MakeNormal MakeDisable
}


#---a line with a label, a entry field and a button
#
proc ::FR_general::labelfieldbutton {Window Row LabelText buttontext} \
{
 label .$Window.label$Row -pady 10 -text $LabelText
 entry .$Window.entry$Row -state normal -width 25 -background white
 button .$Window.button$Row -text $buttontext

 grid .$Window.label$Row -row $Row -column 0 -sticky w
 grid .$Window.entry$Row -row $Row -column 1 -sticky ew
 grid .$Window.button$Row -row $Row -column 2 -sticky ew

 grid columnconfigure .$Window 1 -weight 1
}


#---a line with a label and a entry field
#
proc ::FR_general::labelfield {Window Row LabelText} \
{
 label .$Window.label$Row -pady 10 -text $LabelText
 entry .$Window.entry$Row -state normal -background white

 grid .$Window.label$Row -row $Row -column 0 -sticky w
 grid .$Window.entry$Row -row $Row -column 1 -columnspan 2 -sticky ew

 grid columnconfigure .$Window 1 -weight 1
}


#---a listbox with label and y-scrollbar
#
proc ::FR_general::label_listbox_scrollbar {label_widget listbox_widget scrollbar_widget Row} \
{
 grid $label_widget -row $Row -column 0 -sticky e
 grid $listbox_widget -row $Row -column 2 -sticky ew
 grid $scrollbar_widget -row $Row -column 3 -sticky nsew
}


#---a main topic line
#
proc ::FR_general::headline {Window Row LabelText ColumnSpan {Column 0}} \
{
 label .$Window.label$Row -pady 20 -foreground blue -text $LabelText
 grid .$Window.label$Row -row $Row -column $Column -columnspan $ColumnSpan -sticky w
}


#---a topic line
#
proc ::FR_general::Topic {Window Row LabelText ColumnSpan {Column 0}} \
{
 label .$Window.label$Row -pady 10 -text $LabelText
 grid .$Window.label$Row -row $Row -column $Column -columnspan $ColumnSpan -sticky w
}


#---a line to separate rows
#
proc ::FR_general::Line {Window Row ColumnSpan} \
{
 frame .$Window.line$Row -height 2 -borderwidth 1 -relief sunken
 grid .$Window.line$Row -row $Row -column 0 -columnspan $ColumnSpan -sticky ew
}


#---a output text line with scrollbar
#
proc ::FR_general::output {Window Row TEXT ColumnSpan} \
{
 text .$Window.output$Row -height 1 -wrap none -state normal -background white -xscrollcommand ".$Window.xsbar$Row set"
 scrollbar .$Window.xsbar$Row -orient horizontal -command ".$Window.output$Row xview"
 grid .$Window.output$Row -row $Row -column 0 -columnspan $ColumnSpan -sticky ew
 grid .$Window.xsbar$Row -row [expr {$Row+1}] -column 0 -columnspan $ColumnSpan -sticky nsew
 .$Window.output$Row insert end $TEXT
 .$Window.output$Row configure -state disabled
}

#---a text-widget with scrollbars
#
proc ::FR_general::TextwidgetWithScrollbars {Path {State normal} {Width 50} {Height 12} {Wrap word}} \
{
 set Path [string trimleft $Path .]
 
 text .$Path.output -width $Width -height $Height -wrap $Wrap -background white -state $State -xscrollcommand ".$Path.xsbar set" -yscrollcommand ".$Path.ysbar set"
 scrollbar .$Path.xsbar -orient horizontal -command ".$Path.output xview"
 scrollbar .$Path.ysbar -orient vertical -command ".$Path.output yview"

 grid .$Path.output .$Path.ysbar -sticky nsew
 grid .$Path.xsbar -sticky nsew
 grid columnconfigure .$Path 0 -weight 1
 grid rowconfigure .$Path 0 -weight 1
 
 return "$Path.output"
}


#---a frame to expand a window managed by grid
#
proc ::FR_general::PadFrame {Window Row ColumnSpan {Column 0}} \
{ 
 frame .$Window.padframe$Row -relief flat
  
 grid .$Window.padframe$Row -row $Row -column $Column -columnspan $ColumnSpan -sticky news
 grid rowconfigure .$Window $Row -weight 1
}


#---pack some widgets
#
proc ::FR_general::PackHelp_1 {args} \
{
 # packhelp_1  {Row Widget1 Column Sticky Widget2 Column Sticky...}
 
 set ParameterPerWidget 3
 set WidgetCount [expr {([llength $args] - 1) / $ParameterPerWidget}]
 set Row [lindex $args 0]

 for {set x 1} {$x <= $WidgetCount} {incr x} \
 {
   set Index [expr {${ParameterPerWidget} * ($x-1) + 1}] 
   set Widget [lindex $args $Index]
   set Column [lindex $args [incr Index]]
   set Sticky [lindex $args [incr Index]]
  
   grid $Widget -row $Row -column $Column -sticky $Sticky
 }
}


proc ::FR_general::PackHelp_2 {args} \
{
 # packhelp_2 {Row widget1 column columnspan sticky widget2 column columnspan sticky...}
 set ParameterPerWidget 4
 set WidgetCount [expr {([llength $args] - 1) / $ParameterPerWidget}]
 set Row [lindex $args 0]

 for {set x 1} {$x <= $WidgetCount} {incr x} \
 {
   set Index [expr {${ParameterPerWidget} * ($x-1) + 1}]
   set Widget [lindex $args $Index]
   set Column [lindex $args [incr Index]]
   set ColumnSpan [lindex $args [incr Index]]
   set Sticky [lindex $args [incr Index]]
   grid $Widget -row $Row -column $Column -columnspan $ColumnSpan -sticky $Sticky
 }
}


#---We make a window to a child of a parent window and put it central on the parent window
#---Args: {Child window in the foreground} {parent window in the background} {make it transient to the parent} {desired X of the child window} {desired Y of the child window}
#
proc ::FR_general::CenterWindow {Window {Parent .} {Transient 1} {WindowX 1} {WindowY 1}} \
{
 update
 
 regexp {([0-9]+)x([0-9]+)\+([\-0-9]+)\+([\-0-9]+)} [winfo geometry $Parent] All ParentWidth ParentHeight ParentX ParentY

 #---"All" doesn't exist if "Parent" is minimized
 if {[info exists All]} \
 {
   unset All
 } else \
 {
   # if Parent minimized we place Window in the left-top corner of the screen
   set ParentWidth [winfo reqwidth $Window]
   set ParentHeight [winfo reqheight $Window]
   set ParentX 0
   set ParentY 0
 }
 
 if { ($WindowX == 1) && ($WindowY == 1)} \
 {
   set WindowX [winfo reqwidth $Window]
   set WindowY [winfo reqheight $Window]
 }
 
 set X [expr {($ParentWidth-$WindowX)/2+$ParentX}]
 set Y [expr {($ParentHeight-$WindowY)/2+$ParentY}]
 
 set ScreenHeight [winfo screenheight $Window]
 set ScreenWidth [winfo screenwidth $Window]
  
 # if X or Y are out of the screen -> we correct this
 if {[expr {${WindowX} + $X}] > $ScreenWidth} {set X [expr {${ScreenWidth} - ${WindowX}}]}
 if {[expr {${WindowY} + $Y}] > $ScreenHeight} {set Y [expr {${ScreenHeight} - ${WindowY}}]}
 if {$X < 0} {set X 0}
 if {$Y < 0} {set Y 0}
 
 wm geometry $Window ${WindowX}x$WindowY+$X+$Y
 
 if {$Transient} \
 { 
   wm transient $Window $Parent
 }
}

#---We maximise all handed over windows
#---Args: {Window(s)-path(s)} {0...1 of the screenwith} {0...1 of the screenheight}
#
proc ::FR_general::window_maximise {WINDOWS {PARTX 1} {PARTY 1}} \
{
 update
 
 set SCREENX [winfo screenwidth .]
 set SCREENY [winfo screenheight .]
 
 set SCREENX [expr {round($SCREENX*$PARTX)}]
 set SCREENY [expr {round($SCREENY*$PARTY)}]
 
 foreach WINDOW $WINDOWS \
 {
  wm geometry $WINDOW ${SCREENX}x${SCREENY}+0+0
 }
}

#---we restore all handed over windows
#
proc ::FR_general::window_position_restore {WINDOWS POSITIONS} \
{
 update
 
 foreach WINDOW $WINDOWS POSITION $POSITIONS \
 {
  set ALL ""
  regexp {([0-9]+)x([0-9]+)\+([\-0-9]+)\+([\-0-9]+)} $POSITION ALL WITDTH HEIGHT X Y
  
  if {$ALL ne ""} \
  {
   if {$X < 0} {set X 0}
   if {$Y < 0} {set Y 0}
 
   wm geometry $WINDOW +$X+$Y
  }
 }
}


#---we give back the right width for two or more buttons after translation
#
proc ::FR_general::ButtonsWithSameWidth {ButtonTextList} \
{
  set Width 0
  set ReturnValue ""
  
  foreach Arg $ButtonTextList \
  {
    set Length [string length "$Arg"]
    
    if {$Length > $Width} {set Width $Length}
    
    lappend ReturnValue $Arg
  }
 
  lappend ReturnValue $Width
 
 
  return $ReturnValue
}


#---refresh the width and the height of the scrollable canvas window
proc ::FR_general::ScrollWindowRefresh {Path} \
{
  set Path [string trimleft $Path "."]
  
  update
   
  set WidthCanvas [winfo width .$Path.scrollwindow]
  set HeightCanvas [winfo height .$Path.scrollwindow]
 
  set HeightContent [winfo reqheight .$Path.scrollwindow.content]
 
  # if the content of the canvas window smaller then the canvas window itself we block the scrollbars function with this
  if {$HeightContent <= $HeightCanvas} \
  {
    set BBox [list 0 0 $WidthCanvas $HeightCanvas]
  } \
  else \
  {
    set BBox [list 0 0 $WidthCanvas $HeightContent]
  }
 
  # set the width of the content to the width of the canvas
  .$Path.scrollwindow itemconfigure tag1 -width $WidthCanvas 
  
  # set the scrollregion
  .$Path.scrollwindow configure -scrollregion $BBox
}


#---a scrollwindow with vertical scrollbars
proc ::FR_general::ScrollWindow {Path} \
{
  set Path [string trimleft $Path "."]
  
  scrollbar .$Path.sbary -command ".$Path.scrollwindow yview" -orient vertical
 
  canvas .$Path.scrollwindow -yscrollcommand ".$Path.sbary set" -borderwidth 0 -relief flat
 
  pack .$Path.sbary -side right -fill y
  pack .$Path.scrollwindow -side left -fill both -expand yes
  
  frame .$Path.scrollwindow.content -borderwidth 0
 
  # tag for width setting in "ScrollWindowRefresh"
  .$Path.scrollwindow create window 0 0 -anchor nw -window .$Path.scrollwindow.content -tags tag1

  bind .$Path <Configure> "::FR_general::ScrollWindowRefresh $Path"
 
  return $Path.scrollwindow.content
}


#---the auto-scroll function if a focused widget isn't in the visible area
proc ::FR_general::ScrollWindowShowWidget {Path Widget} \
{
  set Path [string trimleft $Path "."]
  
  # we calculate the y-coordinate of the top and the bottom-corner of the widget and the canvas widget
  set EntryTopCorner [winfo rooty $Widget]
  set EntryBottomCorner [expr {$EntryTopCorner + [winfo height $Widget]}]
  
  set CanvasTopCorner [winfo rooty .${Path}.scrollwindow]
  set CanvasBottomCorner [expr {$CanvasTopCorner + [winfo height .${Path}.scrollwindow]}]

  # if we are out of the visible area we scroll one unit up or down
  if {$EntryTopCorner < $CanvasTopCorner} {.${Path}.scrollwindow yview scroll -1 units}
  if {$EntryBottomCorner > $CanvasBottomCorner} {.${Path}.scrollwindow yview scroll 1 units}
  
  # the validate command of the entry needs this returnvalue (boolean)
  return 0
}


#---configure the scrollwindow increment with help of content widget
proc ::FR_general::ScrollWindowIncrement {Path Widget} \
{
  set Path [string trimleft $Path "."]
  set Widget [string trimleft $Widget "."]
  
  .$Path.scrollwindow configure -yscrollincrement [winfo height .$Widget]
}


#---proc to bind the menu items to a keyboard shortcut
proc ::FR_general::MenuBinding {menupath Position} \
{
  $menupath invoke $Position
  $menupath unpost
  event generate $menupath <KeyPress-Escape>
}


#---set some widgets to normal
proc ::FR_general::MakeNormal {args} \
{
 foreach Widget $args \
 {
  $Widget configure -state normal
 }
}


#---set some widgets to disable
proc ::FR_general::MakeDisable {args} \
{
 foreach Widget $args \
 {
  $Widget configure -state disable
 }
}