# cdripper.tcl

#---the main proc for the cd-ripper
proc ::tk_cdconverter::CdripperMain {} \
{
  variable VAR
   
  ::tk_cdconverter::CdripperGui
  
  ::tk_cdconverter::LibBitrateButtonState $VAR(cdripperframe).bitrate
}


#---after a switch to this program part we make some presettings
proc ::tk_cdconverter::CdripperClearGui {} \
{
  variable VAR
   
  .$VAR(TrackChoiceFrame).trackchoice delete 0 end
  
  ::FR_general::MakeDisable .$VAR(cdripperframe).save .$VAR(cdripperframe).start
}


#---the devicesettings button
proc ::tk_cdconverter::DriveScan {} \
{
  variable CONFIG; variable VAR
 
 # we use this to set the -state of the scan-button
 set VAR(StateScanButton) "disable"
 
 set DriveList ""
 set DriveDescription ""
 
 ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We are looking for your cd-drivers:"]
 
 # if cdda2wav isn't available we can't go further
 if {![::tk_cdconverter::CheckBackgroundTools cdda2wav]} \
 {
   set DriveList [msgcat::mc "none"]
   
   return $DriveList
 }
 
 # cdd2wav -scanbus doesn't work in OpenBSD
 if {$VAR(OpenBSD)} \
 {
   set ScanTool "sysctl"
    
   catch {exec sysctl hw.disknames} DriveInfo
   
   set DriveInfo [regexp -all -inline {cd[0-9]+} $DriveInfo]
   
   foreach Arg $DriveInfo \
   {
     lappend DriveList "/dev/r${Arg}c"
     lappend DriveDescription "/dev/r${Arg}c"
   }
 } \
 else \
 {
   # in all other operation system we are using cdd2wav -scanbus
   set ScanTool "cdda2wav"
     
   catch {eval exec "cdda2wav -scanbus"} DriveInfo
   
   # we search for [Space(s)][Number1],[Number2],[Number3][Space(s)]'[all to the end of line]
   set SearchResult [regexp -all -inline -- {([\s]*)([0-9]+),([0-9]+),([0-9]+)([\s]*)([0-9]+)\)([\s]*)([^\n]*)} $DriveInfo]
   
   set CountOfDrives [expr {[llength $SearchResult] / 9}]
   
   # DriverID is the driver-id: x,x,x; DriverDesc is the name of the driver (cleaned from ' and multiple spaces)
   for {set x 0} {$x < $CountOfDrives} {incr x} \
   {
     set DriverID "[lindex $SearchResult [expr {$x*9+2}]],[lindex $SearchResult [expr {$x*9+3}]],[lindex $SearchResult [expr {$x*9+4}]]"
     set DriverDesc "[lindex $SearchResult [expr {$x*9+8}]]"
      
     set DriverDesc [string trimleft $DriverDesc "'"]
     regsub -all -- {'} $DriverDesc { } DriverDesc
     regsub -all -- {[\s]+} $DriverDesc { } DriverDesc

     # the driver name must have a CD-ROM at the end
     # on my SCSI-device I got an error message (cdda2wav:...) but the device is okay
     if {([regexp {(CD-ROM$)} $DriverDesc]) || ([regexp {(^cdda2wav)} $DriverDesc])} \
     {
       lappend DriveList $DriverID
       lappend DriveDescription "$DriverID: $DriverDesc"
     }
   }    
 }

 
 if {$DriveList eq ""} \
 {
   ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "Sorry, no cd-drivers found?!\nThe output of %s was:\n%s" $ScanTool $DriveInfo]
   set DriveList [msgcat::mc "none"]
 } \
 else \
 {
   ::FR_msgWindow::Add -format Body -msg [msgcat::mc "We found:"]
   
   foreach Arg $DriveDescription \
   {
     ::FR_msgWindow::Add -format Body -msg $Arg
   } 
    
   set VAR(StateScanButton) "normal"
 }
 
 return $DriveList
}


#---Scan the CD and fill the title count into the combo boxes
proc ::tk_cdconverter::ScanCd {} \
{
 variable VAR; variable CONFIG
 
 # if cdda2wav isn't available we can't go further
 if {![::tk_cdconverter::CheckBackgroundTools cdda2wav]} \
 {
   return
 }
 
 if {[::FR_messageBox::Create -message [msgcat::mc "Please put the audio-cd in\nthe CD-ROM drive and press Ok."] -title [msgcat::mc "Content of CD-ROM"] -type Info -button [list [msgcat::mc "Okay"] [msgcat::mc "Cancel"]] ]} \
 {
   set VAR(RipError) 1
   set VAR(NumOfTracks) 0; set VAR(NumOfAudioTracks) 0
         
   ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We read the content of the CD-ROM:"]
          
   set TableOfContain [::FR_exec::Start "cdda2wav --dev=$CONFIG(Device) --no-infofile --info-only --gui -v toc" 20000]
       
   set TableOfContain [lindex $TableOfContain 1]
      
   # we look for all titles (we need the non-audio titles for the freedb access)
   set Titles [regexp -all -inline -- {T([0-9]+):([\s]+)([0-9]+)([\s]+)([0-9]+):([0-9]+).([0-9]+)([^\n]*)} $TableOfContain]
   
   # we calculate the number of titles
   set TitlesLength [llength $Titles]
   set CountPerLine 9
      
   set VAR(NumOfTracks) [expr {$TitlesLength / $CountPerLine}]

   # we generates a list with all sectors
   set VAR(SectorList) ""
   
   if {$VAR(NumOfTracks) > 0} \
   { 
     set AudioTitles [regexp -all -inline -- {T([0-9]+):([\s]+)([0-9]+)([\s]+)([0-9]+):([0-9]+).([0-9]+)([\s]+)audio([^\n]*)} $TableOfContain]
     
     set VAR(NumOfAudioTracks) [expr {[llength $AudioTitles] / 10}]
     
     # we generate a list with all sectors and add the pre-gap (2 seconds (2x75))
     for {set i 3} {$i < $TitlesLength} {incr i $CountPerLine} \
     {
       set SectorStart [lindex $Titles $i]
       
       lappend VAR(SectorList) [expr {$SectorStart + 150}]   
     }
   }
   
   if {$VAR(NumOfAudioTracks) > 0} \
   {
     ::FR_msgWindow::Add -format Body -msg [msgcat::mc "We found: %s audio tracks" $VAR(NumOfAudioTracks)]
     
     set VAR(RipError) 0
     
     ::tk_cdconverter::AddTracks
   } \
   else \
   {
     ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "We didn't found any audio tracks on this cd."]
     ::FR_msgWindow::Add -format Body -msg [msgcat::mc "The output of cdda2wav is:\n%s" $TableOfContain]
   }
   
   # we calculate the time for the whole cd
   if {[regexp {Tracks:([0-9]+)([\s]+)([0-9]+):([0-9]+).([0-9]+)} $TableOfContain All Tracks Space Minutes Seconds mSeconds]} \
   {
     # convert the strings to numeric values (e.g. from 02 to 2)
     set Minutes [expr {int($Minutes)}]
     set Seconds [expr {int($Seconds)}]
     set mSeconds [expr {int($mSeconds)}]
 
     set VAR(CdTime) [expr {($Minutes * 60) + $Seconds + ($mSeconds / 100)}]
   } \
   else \
   {
     set VAR(CdTime) 0
   }
 }
}


#---add a number for each track into the listbox
proc ::tk_cdconverter::AddTracks {}\
{
 variable VAR
 
 .$VAR(TrackChoiceFrame).trackchoice delete 0 end
 
 for {set x 1} {$x <= $VAR(NumOfAudioTracks)} {incr x} \
 {
   # we fill the track choise listbox
   .$VAR(TrackChoiceFrame).trackchoice insert end $x
 }
}


#---the reaction after selecting tracks in the listbox
proc ::tk_cdconverter::TrackSelection {Widget {Command ""}} \
{
 variable VAR
  
 set VAR(SelectedTracks) ""
 
 if {$Command eq "DeselectAll"} \
 {
   .$VAR(TrackChoiceFrame).trackchoice selection clear 0 end
 } \
 elseif {$Command eq "SelectAll"} \
 {
   .$VAR(TrackChoiceFrame).trackchoice selection set 0 end
 }
 
  
 set Index [$Widget curselection]
  
 foreach i $Index \
 {
   lappend VAR(SelectedTracks) [$Widget get $i]
 }
    
 if { $VAR(SelectedTracks) eq "" } \
 {
   .$VAR(cdripperframe).save configure -state disable
 } \
 else \
 { 
   .$VAR(cdripperframe).save configure -state normal 
 }
  
 ::FR_msgWindow::Add -format Info -msg [msgcat::mc "You selected track(s): %s" $VAR(SelectedTracks)]
}


#---the "save where" button
proc ::tk_cdconverter::SaveOutputFiles {} \
{
 variable VAR; variable CONFIG
  
 set OutputDir [::FR_dirBox::Create -initialdir $CONFIG(OutputDir) -title [msgcat::mc "Choose the directory for your output files:"]  -permission "writable" -filetype $VAR(EncoderFileExtension)]
 
 if {$OutputDir ne ""} \
 { 
   set CONFIG(OutputDir) $OutputDir
   
   ::FR_msgWindow::Add -format Info -msg [msgcat::mc "The outputfile directory is set to %s" $OutputDir]
   
   .$VAR(cdripperframe).start configure -state normal
 } 
}


#---we create and place all gui elements
proc ::tk_cdconverter::CdripperGui {} \
{
 variable CONFIG; variable VAR
 
 # the widgets
 label .$VAR(cdripperframe).device_comment -pady 10 -text [msgcat::mc "Ripping-device:"]
 ::FR_comboBox::cb .$VAR(cdripperframe).device -variable ::tk_cdconverter::CONFIG(Device) -entry_width auto
 ::FR_comboBox::cb .$VAR(cdripperframe).device -listbox_content [::tk_cdconverter::DriveScan] -default_value $::tk_cdconverter::CONFIG(Device)
  
 label .$VAR(cdripperframe).scan_comment -pady 10 -text [msgcat::mc "Scan the content of the CD (TOC):"]
 button .$VAR(cdripperframe).scan -text [msgcat::mc "Check"] -state $::tk_cdconverter::VAR(StateScanButton) -command {::tk_cdconverter::ScanCd}
 
 frame .$VAR(cdripperframe).trackchoiceframe 
 set VAR(TrackChoiceFrame) "${VAR(cdripperframe)}.trackchoiceframe"
 checkbutton .$VAR(cdripperframe).live -text [msgcat::mc "All selected tracks in one file (live recording)?"] -variable ::tk_cdconverter::CONFIG(LiveRecording) -pady 10
 
 label .$VAR(cdripperframe).outputformat_comment -pady 10 -text [msgcat::mc "Outputformat:"]
 ::FR_comboBox::cb .$VAR(cdripperframe).outputformat -variable ::tk_cdconverter::CONFIG(OutputFormat) -entry_width auto
 ::FR_comboBox::cb .$VAR(cdripperframe).outputformat -listbox_content $::tk_cdconverter::VAR(EncoderList) -default_value $::tk_cdconverter::CONFIG(OutputFormat) -command [list ::tk_cdconverter::LibBitrateButtonState $VAR(cdripperframe).bitrate]
 
 label .$VAR(cdripperframe).bitrate_comment -text [msgcat::mc "Select the desired bitrate (mp3/ogg-vorbis only):"]

 ::FR_comboBox::cb .$VAR(cdripperframe).bitrate -variable ::tk_cdconverter::CONFIG(BitRate)
 ::FR_comboBox::cb .$VAR(cdripperframe).bitrate -listbox_content [list 64 128 192 256] -entry_width 3 -default_value $::tk_cdconverter::CONFIG(BitRate)
 
 label .$VAR(cdripperframe).save_comment -pady 10 -text [msgcat::mc "Save the output files to:"]
 button .$VAR(cdripperframe).save -text [msgcat::mc "Save"]  -state disabled -command {::tk_cdconverter::SaveOutputFiles}
 
 button .$VAR(cdripperframe).start -text [msgcat::mc "Start ripping"] -foreground blue -state disabled -command {::tk_cdconverter::CdripperStartButton}
 
 # the placement
 set row 0
 eval ::FR_general::PackHelp_1 $row .$VAR(cdripperframe).device_comment 0 w .$VAR(cdripperframe).device 4 e
 ::FR_general::Line $VAR(cdripperframe) [incr row] 5
 
 eval ::FR_general::PackHelp_1 [incr row] .$VAR(cdripperframe).scan_comment 0 w .$VAR(cdripperframe).scan 4 ew
 ::FR_general::Line $VAR(cdripperframe) [incr row] 5
 
 ::FR_general::Topic $VAR(cdripperframe) [incr row] [msgcat::mc "Select the titles which you want to rip:"] 5
 eval ::FR_general::PackHelp_2 [incr row] .$VAR(TrackChoiceFrame) 0 5 ew
 eval ::FR_general::PackHelp_2 [incr row] .$VAR(cdripperframe).live 0 5 w
 ::FR_general::Line $VAR(cdripperframe) [incr row] 5
 
 eval ::FR_general::PackHelp_1 [incr row] .$VAR(cdripperframe).outputformat_comment 0 w .$VAR(cdripperframe).outputformat 4 ew
 eval ::FR_general::PackHelp_1 [incr row] .$VAR(cdripperframe).bitrate_comment 0 w .$VAR(cdripperframe).bitrate 4 ew
 eval ::FR_general::PackHelp_1 [incr row] .$VAR(cdripperframe).save_comment 0 w .$VAR(cdripperframe).save 4 ew
 
 ::FR_general::Line $VAR(cdripperframe) [incr row] 5
 ::FR_general::PadFrame $VAR(cdripperframe) [incr row] 5
 eval ::FR_general::PackHelp_2 [incr row] .$VAR(cdripperframe).start 0 5 ew
 
 grid columnconfigure .$VAR(cdripperframe) 3 -weight 1
 
 # the widgets of the TrackChoiceFrame
 listbox .$VAR(TrackChoiceFrame).trackchoice -background white -activestyle none -selectmode extended -yscroll ".$VAR(TrackChoiceFrame).yscrollbar set" -height 5 
 scrollbar .$VAR(TrackChoiceFrame).yscrollbar -orient vertical -command ".$VAR(TrackChoiceFrame).trackchoice yview" 
 bind .$VAR(TrackChoiceFrame).trackchoice <<ListboxSelect>> {::tk_cdconverter::TrackSelection %W}

 button .$VAR(TrackChoiceFrame).selectall -text [msgcat::mc "Select all"] -command [list ::tk_cdconverter::TrackSelection .$VAR(TrackChoiceFrame).trackchoice "SelectAll"] 
 button .$VAR(TrackChoiceFrame).deselect -text [msgcat::mc "Deselect all"] -command [list ::tk_cdconverter::TrackSelection .$VAR(TrackChoiceFrame).trackchoice "DeselectAll"]
    
 # the placement of the TrackChoiceFrame
 set row 0  
 grid .$VAR(TrackChoiceFrame).trackchoice -row $row -column 0 -columnspan 2 -sticky nsew
 grid .$VAR(TrackChoiceFrame).yscrollbar -row $row -column 2 -sticky nsw
 
 grid .$VAR(TrackChoiceFrame).selectall -row [incr row] -column 0 -sticky ew
 grid .$VAR(TrackChoiceFrame).deselect -row $row -column 1 -sticky ew
  
 grid columnconfigure .$VAR(TrackChoiceFrame) 3 -weight 1
 grid rowconfigure .$VAR(TrackChoiceFrame) 0 -weight 1
 
 # the balloon help
 ::FR_balloonHelp::AddHelp ".$VAR(cdripperframe).device" [msgcat::mc "Device name of your CD-ROM drive checked with: \"cdda2wav -scanbus\" or \"sysctl hw.disknames\" (at OpenBSD)"]
 ::FR_balloonHelp::AddHelp ".$VAR(cdripperframe).scan" [msgcat::mc "First scan the content of the audio-cd"]
 ::FR_balloonHelp::AddHelp ".$VAR(cdripperframe).save" [msgcat::mc "This is the directory in which you want to rip the cd-tracks"]
 ::FR_balloonHelp::AddHelp ".$VAR(TrackChoiceFrame).trackchoice" [msgcat::mc "Select the desired tracks (for multiple selecting you can use the Shift- or Ctrl-key)"]
 ::FR_balloonHelp::AddHelp ".$VAR(cdripperframe).live" [msgcat::mc "We rip all chosen tracks in one file (without interruptions)"]
 
 ::tk_cdconverter::LibBalloonHelpStatus
}


#---the start button
proc ::tk_cdconverter::CdripperStartButton {} \
{
 variable VAR; variable CONFIG
 
 ::FR_general::MakeDisable .$VAR(cdripperframe).save .$VAR(cdripperframe).start
  
 # nothing selected - nothing to do...
 if {$VAR(SelectedTracks) eq ""} \
 {
   ::FR_msgWindow::Add -format Error -msg [msgcat::mc "There arn't any tracks selected. I can't do anythink for you... :-("]
   return 1
 } 
 
 
 if {$VAR(RipError) != "0"} \
 {
   ::FR_messageBox::Create -message [msgcat::mc "I can't read the audio-cd...?!\nPlease check the message window"] -title [msgcat::mc "Uuups!!"] -type Error
 } \
 else \
 {      
   cd $CONFIG(OutputDir)
   
   set ReturnValue [::tk_cdconverter::CreateFileNames]
   
   if {$ReturnValue == 0} \
   {
     set ReturnValue [::tk_cdconverter::Rip]
   }
   
   if {($ReturnValue == 0)} \
   {
     set ReturnValue [::tk_cdconverter::CdripperConverter]  
   }
   
   if {$ReturnValue == 0} \
   {
     ::FR_messageBox::Create -message [msgcat::mc "We have ripped all tracks successfully"] -title [msgcat::mc "End of job"] -type Info
   } \
   else \
   {
     ::FR_messageBox::Create -message [msgcat::mc "We couldn't finish this job successfully.\n\nPlease check the message window for further informations."] -type Error
   }   
 }
 
 # we delete two cdda2wav info file which aren't needed
 foreach Arg [list [file join $CONFIG(OutputDir) audio.cddb] [file join $CONFIG(OutputDir) audio.cdindex]] \
 {
   if {[file exists $Arg]} \
   {
     catch {file delete $Arg}
   }
 }
 
 .$VAR(TrackChoiceFrame).trackchoice selection clear 0 end
}


#---we check for existing files and create a file list
proc ::tk_cdconverter::CreateFileNames {}\
{
  variable CONFIG; variable VAR

  set ExistingFiles [::tk_cdconverter::LibCheckDirContent $CONFIG(OutputDir) $CONFIG(OutputFormat)]
  
  if {$ExistingFiles == ""} \
  {
    set Index 0
    set ExistingFiles 0
  } \
  else \
  {
    set Index [llength $ExistingFiles]
    set ExistingFiles 1
  }
  
  set VAR(CdripperFileList) ""
  set OutputFileList ""
  
  foreach Track $VAR(SelectedTracks) \
  {
    incr Index
    
    set OutputFileName [::tk_cdconverter::LibCreateFileName "Song" $Index $CONFIG(OutputFormat)]  
  
    while {[file exists $OutputFileName]} \
    {
      incr Index
      
      set OutputFileName [::tk_cdconverter::LibCreateFileName "Song" $Index $CONFIG(OutputFormat)]
    }
  
    set WavFileName [::tk_cdconverter::LibCreateFileName "SongTmp" $Index "Wav"]
  
    lappend VAR(CdripperFileList) $Track $Index $WavFileName $OutputFileName
    lappend OutputFileList $OutputFileName
  }
  
  if {$ExistingFiles} \
  {
    set Answer [::FR_messageBox::Create -message [msgcat::mc "There are already %s-files in the output directory.\n\nWe have looked for unused file names and suggest the following names:\n\n%s" $CONFIG(OutputFormat) $OutputFileList] -title [msgcat::mc "Other files found"] -type Info -button [list [msgcat::mc "Accept"] [msgcat::mc "Abort"]] ]
   
    if {$Answer == 0} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Abort was your decision, we stoped the job."] 
      return 1
    }
  }
 
  return 0
}


#---the ripping procedures
proc ::tk_cdconverter::Rip {}\
{
 variable CONFIG; variable VAR
 
 set ReturnValue 1

 # now we can really start to generate wav file(s)
 if {$CONFIG(LiveRecording) == 1} \
 {
   set ReturnValue [::tk_cdconverter::RipLiveTrack [lindex $VAR(CdripperFileList) 0] [lindex $VAR(CdripperFileList) end-3] [lindex $VAR(CdripperFileList) 2]]
 } \
 else \
 {
   foreach "Track Index WavFileName OutputFileName" $VAR(CdripperFileList) \
   {
     set ReturnValue [::tk_cdconverter::RipSingleTrack $Track $WavFileName]
     
     if {$ReturnValue} \
     {
       break;
     }
   }
 }
 
 return $ReturnValue
}


#---sub procedure for tracks ripping
proc ::tk_cdconverter::RipSingleTrack {Track FileName}\
{
 variable CONFIG; variable VAR
 
 ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We rip the %s. track!" $Track]
 
 set ReturnValue [::FR_exec::Start "cdda2wav --dev=$CONFIG(Device) -t $Track -x [file join $CONFIG(OutputDir) $FileName]"]
 #-paranoia --no-infofile
 
 if {[lindex $ReturnValue 0] == 0} \
 {
   ::FR_msgWindow::Add -format Body -msg [msgcat::mc "Track successful ripped!"]   
   return 0
 } \
 else \
 {
   ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "An error occured:\n%s" [lindex $ReturnValue 1]]
   return 1   
 }
}


#---sub procedure for live ripping
proc ::tk_cdconverter::RipLiveTrack {StartTrack LastTrack FileName}\
{
 variable CONFIG; variable VAR
 
 ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We are ripping the live-recording form %s to %s!" $StartTrack $LastTrack]

 set ReturnValue [::FR_exec::Start "cdda2wav --dev=$CONFIG(Device) --no-infofile -t ${StartTrack}+${LastTrack} -x [file join $CONFIG(OutputDir) $FileName]"]
 #-paranoia
 
 if {[lindex $ReturnValue 0] == 0} \
 {
   ::FR_msgWindow::Add -format Body -msg [msgcat::mc "Track successful ripped!"]
   return 0
 } \
 else \
 {
   ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "An error occured:\n%s" [lindex $ReturnValue 1]]
   return 1
 }
}


#---we convert the ripped wave files to the output format
proc ::tk_cdconverter::CdripperConverter {} \
{
  variable VAR; variable CONFIG
  
  set ReturnValue 1
    
  foreach "Track Index WavFileName OutputFileName" $VAR(CdripperFileList) \
  {
    ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We convert the %s. track!" $Track]
    
    set ReturnValue [::tk_cdconverter::LibWav2Compressed $WavFileName $OutputFileName $CONFIG(OutputFormat) $CONFIG(BitRate)]
    
    if {$ReturnValue != 0} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I couldn't generate the output file(s). The reason for that was:\n%s" $ReturnValue]
      
      set ReturnValue 1
      break
    }
    
    set ReturnValue [::tk_cdconverter::RenameInfFiles $WavFileName $OutputFileName]
    
    if {$ReturnValue != 0} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I couldn't generate the file with the track informations. The reason for that was:\n%s" $ReturnValue]
      
      set ReturnValue 1
      break
    }
        
    if {[file exists $WavFileName]} \
    {
      catch {file delete $WavFileName}
    }
    
    ::FR_msgWindow::Add -format Body -msg [msgcat::mc "Track successful converted! The name of the output file is: %s" $OutputFileName]
  }
  
  return $ReturnValue
}


#---we rename the inf-files to be the same as the outputfile name
proc ::tk_cdconverter::RenameInfFiles {WavFileName OutputFileName} \
{
  variable VAR
  
  set WavInfName [file rootname $WavFileName]
  append WavInfName ".inf"
  
  set OutputInfName [file rootname $OutputFileName]
  append OutputInfName ".inf"
  
  set ReturnValue [file rename -force $WavInfName $OutputInfName]
  
  if {$ReturnValue == ""} \
  {
    set ReturnValue 0
    
    if {[file exists $OutputInfName] && [file writable $OutputInfName]} \
    {
      set Fid [open $OutputInfName a]
    
      puts $Fid "# Added by tk_cdconverter"
      puts $Fid "Tracks: $VAR(NumOfTracks)"
      puts $Fid "Sectors: $VAR(SectorList)"
      puts $Fid "Cd-Time: $VAR(CdTime)"
    
      close $Fid
    }
  }
    
  return $ReturnValue 
}


::tk_cdconverter::CdripperMain
