# in this file are functions which are needed in more than one part of tk_cdconverter

#---switch on/off the balloonhelp
proc ::tk_cdconverter::LibBalloonHelpStatus {} \
{
  variable CONFIG
  
  if {$CONFIG(BalloonHelp)} \
  {
    ::FR_balloonHelp::Activate
  } else \
  {
    ::FR_balloonHelp::Deactivate
  }
}


#---a list with the different genres and its numbers
proc ::tk_cdconverter::LibGenerateGenreList {} \
{
  variable VAR
 
  set VAR(GenreList) [list 123 "A Cappella" 34 Acid 74 "Acid Jazz" 73 "Acid Punk" 99 Acoustic 20 Alternative 40 "Alt. Rock" 26 Ambient 145 Anime 90 Avantgarde 116 Ballad 41 Bass 135 Beat 85 Bebob 96 "Big Band" 138 "Black Metal" 89 Bluegrass 0 Blues 107 "Booty Bass" 132 BritPop 65 Cabaret 88 Celtic 104 "Chamber Music" 102 Chanson 97 Chorus 136 "Christian Gangsta Rap" 61 "Christian Rap" 141 "Christian Rock" 32 Classical  1 "Classic Rock" 112 Club 128 "Club-House" 57 Comedy 140 "Contemporary Christian" 2 Country 139 Crossover 58 Cult 3 Dance 125 "Dance Hall" 50 Darkwave 22 "Death Metal" 4 Disco 55 Dream 127 "Drum & Bass" 122 "Drum Solo" 120 Duet 98 "Easy Listening" 52 Electronic 48 Ethnic 54 Eurodance 124 "Euro-House" 25 "Euro-Techno" 84 "Fast-Fusion" 80 Folk 115 Folklore 81 "Folk/Rock" 119 Freestyle 5 Funk 30 Fusion 36 Game 59 "Gangsta Rap" 126 Goa 38 Gospel 49 Gothic 91 "Gothic Rock"  6 Grunge 129 Hardcore 79 "Hard Rock" 137 "Heavy Metal" 7 "Hip-Hop" 35 House 100 Humour 131 Indie 19 Industrial 33 Instrumental 46 "Instrumental Pop" 47 "Instrumental Rock"  8 Jazz 29 "Jazz+Funk" 146 JPop 63 Jungle 86 Latin 71 "Lo-Fi" 45 Meditative 142 Merengue 9 Metal 77 Musical 82 "National Folk" 64 "Native American" 133 Negerpunk 10 "New Age" 66 "New Wave" 39 Noise 11 Oldies 103 Opera 12 Other 75 Polka 134 "Polsk Punk" 13 Pop 53 "Pop-Folk" 62 "Pop/Funk" 109 "Porn Groove" 117 "Power Ballad" 23 Pranks 108 Primus 92 "Progressive Rock" 67 Psychedelic 93 "Psychedelic Rock" 43 Punk 121 "Punk Rock" 15 Rap 68 Rave 14 "R&B" 16 Reggae 76 Retro 87 Revival 118 "Rhythmic Soul" 17 Rock 78 "Rock & Roll" 143 Salsa 114 Samba 110 Satire 69 Showtunes 21 Ska 111 "Slow Jam" 95 "Slow Rock" 105 Sonata 42 Soul 37 "Sound Clip" 24 Soundtrack 56 "Southern Rock" 44 Space 101 Speech 83 Swing 94 "Symphonic Rock" 106 Symphony 147 Synthpop 113 Tango 18 Techno 51 "Techno-Industrial" 130 Terror 144 "Thrash Metal" 60 "Top 40" 70 Trailer 31 Trance 72 Tribal 27 "Trip-Hop" 28 Vocal]
 
  set VAR(GenreListLength) [llength $VAR(GenreList)]
  
  set VAR(GenreListTxt) ""
  
  for {set i 1} {$i < $VAR(GenreListLength)} {incr i 2} \
  {
    lappend VAR(GenreListTxt) [lindex $VAR(GenreList) $i]
  }
}


#---read out the tag and return all values
proc ::tk_cdconverter::LibReadTag {File FileFormat} \
{
  variable VAR; variable CONFIG
  
  switch $FileFormat \
  {
    "Mp3"        { if {($VAR(id3info)) && $CONFIG(UseId3v2)} \
                   {
                     # in this case we can read id3v2
                     set ReturnValue [::tk_cdconverter::LibReadId3v2Tag $File]
                   } \
                   else \
                   {
                     # in the other case only id3v1
                     set ReturnValue [::tk_cdconverter::LibReadId3v1Tag $File]
                   }
                 } \
    "Ogg-vorbis" {set ReturnValue [::tk_cdconverter::LibReadVorbisTag $File]} \
    "Flac"       {set ReturnValue [::tk_cdconverter::LibReadFlacTag $File]} \
    default      {set ReturnValue [::tk_cdconverter::LibReadUnknownTag $File]} \
  }

  return $ReturnValue
}


#---if we don't know the format we give back an empty list
proc ::tk_cdconverter::LibReadUnknownTag {File} \
{
  foreach Arg [list Title Artist Year Album Track] \
  {
    set $Arg ""
  }

  # we set genre to the default value "Other"
  set Genre "Other"
  
  set FileName [file rootname $File]
  append FileName "Tags.txt"
    
  if {[file exists $FileName] && [file readable $FileName]} \
  {
    set Fid [open $FileName r]
    set FileContent [read $Fid]
    close $Fid
  } \
  else \
  {
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
  
  if {$FileContent != ""} \
  {
    regexp {Title: ([^\n]*)} $FileContent All Title
    regexp {Artist: ([^\n]*)} $FileContent All Artist
    regexp {Year: ([^\n]*)} $FileContent All Year
    regexp {Album: ([^\n]*)} $FileContent All Album
    regexp {Genre: ([^\n]*)} $FileContent All Genre
    regexp {Track: ([^\n]*)} $FileContent All Track  
  }
  
  return [list $Title $Artist $Year $Album $Genre $Track]
}


#---we read all informations from the mp3 file
proc ::tk_cdconverter::LibReadId3v1Tag {File} \
{
  foreach Arg [list Title Artist Year Album Track] \
  {
    set $Arg ""
  }
  
  # we set genre to the default value "Other"
  set Genre "Other"
  
  if {![file readable $File]} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "The file %s isn't readable, we must abort." $File]
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
  
  set Fid [open $File r]
  fconfigure $Fid -encoding binary -translation binary
  
  # if the file lower than 128Byte, it coundn't be a mp3 file
  if {[catch {seek $Fid -128 end}]} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "%s must be corrupted (size < 128Byte)" $File]
    set Info "Error"
  } \
  else \
  {  
    set Info [read $Fid]
  }
 
  close $Fid
         
  # has the file an error we return the empty default values
  if {$Info eq "Error"} \
  {
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
   
  binary scan $Info A3 Id

  if {[regexp -nocase -- {tag} $Id]} \
  {
    binary scan $Info A3A30A30A30A4A28ccc Id Title Artist Album Year comment zero Track Genre
  } \
  else \
  {
    # if we don't find the tag field we return the empty default values
    return [list $Title $Artist $Year $Album $Genre]
  }
  
  set Genre [::tk_cdconverter::LibGenreId2GenreTxt $Genre]
  
  # we remove all leading and trailing spaces
  foreach Arg [list Title Artist Year Album Track] \
  {
    set $Arg [string trim [set $Arg]]
  }
  
  return [list $Title $Artist $Year $Album $Genre $Track]
}


#---execute and parse the output of id3info
proc ::tk_cdconverter::LibReadId3v2Tag {File} \
{
  variable VAR
  
  foreach Arg [list Title Artist Year Album Genre Track] \
  {
    set $Arg ""
  }
  
  set ReturnValue [catch {exec id3info $File} Info]
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't read the tag. The reason for this was:\n%s" $Info]
    
    # in the error case we return an empty list
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
   
  if {$Info != ""} \
  {
    regexp {TIT2 \(Title/songname/content description\): ([^\n]*)} $Info All Title
    regexp {TPE1 \(Lead performer\(s\)/Soloist\(s\)\): ([^\n]*)} $Info All Artist
    regexp {TYER \(Year\): ([^\n]*)} $Info All Year
    regexp {TCON \(Content type\): \(([^\)]*)} $Info All Genre
    regexp {TALB \(Album/Movie/Show title\): ([^\n]*)} $Info All Album
    regexp {TRCK \(Track number/Position in set\): ([^\n]*)} $Info All Track
    
    # Track could be in the form 1/10 (1 from 10), we isolate the track
    set Tmp [split $Track /]
    set Track [lindex $Tmp 0]
    
    # we remove all leading and trailing spaces
    foreach Arg [list Title Artist Year Genre Album Track] \
    {
      set $Arg [string trim [set $Arg]]
    }
  }
  
  set Genre [::tk_cdconverter::LibGenreId2GenreTxt $Genre]
  
  return [list $Title $Artist $Year $Album $Genre $Track]
}


#---we read out the vorbiscomment 
proc ::tk_cdconverter::LibReadVorbisTag {File} \
{
  variable VAR
    
  foreach Arg [list Title Artist Year Album Genre Track] \
  {
    set $Arg ""
  }
  
  set ReturnValue [catch {exec vorbiscomment -l $File} Info]
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't read the tag. The reason for this was:\n%s" $Info]
    
    # in the error case we return an empty list
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
  
  if {$Info != ""} \
  {
    regexp -nocase -- {artist=([^\n]*)} $Info All Artist
    regexp -nocase -- {genre=([^\n]*)} $Info All Genre
    regexp -nocase -- {date=([^\n]*)} $Info All Year
    regexp -nocase -- {title=([^\n]*)} $Info All Title
    regexp -nocase -- {album=([^\n]*)} $Info All Album
    regexp -nocase -- {tracknumber=([^\n]*)} $Info All Track
    
    # we remove all leading and trailing spaces
    foreach Arg [list Title Artist Year Genre Album Track] \
    {
      set $Arg [string trim [set $Arg]]
    }
  }
 
  if {$Genre eq ""} \
  {
    set Index "-1"
  } \
  else \
  {
    set Index [lsearch -exact $VAR(GenreList) $Genre]
  }
  
  if {$Index == "-1"} \
  {
    set Genre "Other"
  }
  
  return [list $Title $Artist $Year $Album $Genre $Track]
}


#---we read out the tag with metaflac
proc ::tk_cdconverter::LibReadFlacTag {File} \
{
  variable VAR
    
  foreach Arg [list Title Artist Year Album Genre Track] \
  {
    set $Arg ""
  }
  
  set ReturnValue [catch {exec metaflac --list -- $File} Info]
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't read the tag. The reason for this was:\n%s" $Info]
    
    # in the error case we return an empty list
    return [list $Title $Artist $Year $Album $Genre $Track]
  }
  
  if {$Info != ""} \
  {
    regexp -nocase -- {artist=([^\n]*)} $Info All Artist
    regexp -nocase -- {genre=([^\n]*)} $Info All Genre
    regexp -nocase -- {date=([^\n]*)} $Info All Year
    regexp -nocase -- {title=([^\n]*)} $Info All Title
    regexp -nocase -- {album=([^\n]*)} $Info All Album
    regexp -nocase -- {tracknumber=([^\n]*)} $Info All Track
    
    # we remove all leading and trailing spaces
    foreach Arg [list Title Artist Year Genre Album Track] \
    {
      set $Arg [string trim [set $Arg]]
    }
  }

  if {$Genre eq ""} \
  {
    set Index "-1"
  } \
  else \
  {
    set Index [lsearch -exact $VAR(GenreList) $Genre]
  }
  
  if {$Index == "-1"} \
  {
    set Genre "Other"
  }
  
  return [list $Title $Artist $Year $Album $Genre $Track]
}


#---convertes: genre-id -> genre-text 
proc ::tk_cdconverter::LibGenreId2GenreTxt {Value} \
{
  variable VAR
 
  # if value isn't a number we set the genre-txt to "Other"
  if {![regexp {[0-9]+} $Value]} \
  {
    return "Other" 
  }
  
  # we convert the genre-number in the genre-text
  set Index [lsearch -exact $VAR(GenreList) $Value]

  if {$Index == "-1"} \
  {
    # if we don't find anything we set the genre-txt to "Other"
    return "Other"
  } \
  else \
  {
    return [lindex $VAR(GenreList) [incr Index]]
  }
}


#---convertes: genre-text -> genre-id
proc ::tk_cdconverter::LibGenreTxt2GenreId {Value} \
{
  variable VAR
 
  # we convert the genre-text in the genre-number
  set Index [lsearch -exact $VAR(GenreList) $Value]
                    
  if {$Index == "-1"} \
  {
    # if we don't find anything we set the genre-id to "Other"
    return "12" 
  } \
  else \
  {
    return [lindex $VAR(GenreList) [incr Index -1]]
  }
}


#---read out the tag and return all values
proc ::tk_cdconverter::LibWriteTag {Title Artist Year Album Genre Track File FileFormat} \
{
  variable VAR; variable CONFIG
  
  switch $FileFormat \
  {
    "Mp3"        {if {($VAR(id3tag)) && $CONFIG(UseId3v2)} \
                   {
                     # in this case we can write id3v2
                     set ReturnValue [::tk_cdconverter::LibWriteId3v2Tag $Title $Artist $Year $Album $Genre $Track $File] 
                   } \
                   else \
                   {
                     # in the other case only id3v1
                     set ReturnValue [::tk_cdconverter::LibWriteId3v1Tag $Title $Artist $Year $Album $Genre $Track $File]
                   }
                 } \
    "Ogg-vorbis" {set ReturnValue [::tk_cdconverter::LibWriteVorbisTag $Title $Artist $Year $Album $Genre $Track $File]} \
    "Flac"       {set ReturnValue [::tk_cdconverter::LibWriteFlacTag $Title $Artist $Year $Album $Genre $Track $File]} \
    default      {set ReturnValue [::tk_cdconverter::LibWriteUnknownTag $Title $Artist $Year $Album $Genre $Track $File]} \
  }
  
  return $ReturnValue
}


#---if the file format is unknown (or without a tag possibility) we save all informations in a txt file
proc ::tk_cdconverter::LibWriteUnknownTag {Title Artist Year Album Genre Track File} \
{
  variable VAR
  
  set FileName [file rootname $File]
  append FileName "Tags.txt"
    
  # we write a file with all tags
  if {[catch {set Fid [open $FileName w]}] == 0} \
  { 
    puts $Fid "# tk_cdconverter"
    puts $Fid "#"
    puts $Fid "# Your target compressed format was unknown, so we saved all tags in this file"
    
    puts $Fid "Title: $Title"     
    puts $Fid "Artist: $Artist"
    puts $Fid "Year: $Year"
    puts $Fid "Album: $Album"
    puts $Fid "Genre: $Genre"
    puts $Fid "Track: $Track"
    
    close $Fid
    
    set ReturnValue 0
  } \
  else \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We can't open the file %s, we must abort." $FileName]
    set ReturnValue 1
  }
  
  return $ReturnValue
}  
  

#---we write all informations to the mp3 file
proc ::tk_cdconverter::LibWriteId3v1Tag {Title Artist Year Album Genre Track File} \
{
  variable VAR
 
  if {![file writable $File]} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "The file %s isn't writable, we must abort." $File]
    return 1
  }
  
  set Fid [open $File a+]
  
  fconfigure $Fid -encoding binary -translation binary
  
  # if the file lower than 128Byte, it coundn't be an mp3 file
  
  if {[catch {seek $Fid -128 end}]} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "%s must be corrupted (size < 128Byte)" $File]
    return 1
  } \
  else \
  {  
    set Tag [read $Fid]
  }
   
  binary scan $Tag A3 Id
  
  # if we didn't find an old tag we append the new tag at the end of the file
  # in the other case we overwrite the old tag
  if {![regexp -nocase -- {TAG} $Id]} \
  {
    catch {seek $Fid 0 end}
  }
  
  set Genre [::tk_cdconverter::LibGenreTxt2GenreId $Genre]
  
  set NewTag [binary format a3a30a30a30a4a28ccc TAG $Title $Artist $Album $Year $::tk_cdconverter::VAR(ConverterComment) 0 $Track $Genre]
    
  puts -nonewline $Fid $NewTag
  
  close $Fid
  
  return 0
}


#---we write all informations to the ogg files
proc ::tk_cdconverter::LibWriteId3v2Tag {Title Artist Year Album Genre Track File} \
{
  variable VAR
  
  set Genre [::tk_cdconverter::LibGenreTxt2GenreId $Genre]
  
  set ReturnValue [catch {exec id3tag --artist=$Artist --album=$Album --song=$Title --year=$Year --track=$Track --genre=$Genre --comment=$VAR(ConverterComment) $File} CatchOutput]
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't write the tag. The reason for this was:\n%s" $CatchOutput]
  }
  
  return $ReturnValue
}


#---we write all informations to the ogg files
proc ::tk_cdconverter::LibWriteVorbisTag {Title Artist Year Album Genre Track File} \
{
  variable VAR
  
  set ReturnValue [catch {exec vorbiscomment -w -t Artist=$Artist -t Album=$Album -t Title=$Title -t Date=$Year -t Tracknumber=$Track -t Genre=$Genre -t Comment=$VAR(ConverterComment) $File} CatchOutput]
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't write the tag. The reason for this was:\n%s" $CatchOutput]
  }
  
  return $ReturnValue
}


#---we write all informations to the flac files
proc ::tk_cdconverter::LibWriteFlacTag {Title Artist Year Album Genre Track File} \
{
  variable VAR

  set ReturnValue [catch {exec metaflac --remove-all-tags $File} CatchOutput]
  
  if {$ReturnValue == 0} \
  {
    set ReturnValue [catch {exec metaflac --set-tag=Artist=$Artist --set-tag=Album=$Album --set-tag=Title=$Title --set-tag=Date=$Year --set-tag=Tracknumber=$Track --set-tag=Genre=$Genre --set-tag=Comment=$VAR(ConverterComment) $File} CatchOutput]
  }
  
  if {$ReturnValue != 0} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't write the tag. The reason for this was:\n%s" $CatchOutput]
  }
  
  return $ReturnValue
}


#---returns the sorted content of a directory for a given file type
proc ::tk_cdconverter::LibCheckDirContent {Directory Format} \
{
  cd $Directory
 
  switch $Format \
  {
    "Ogg-vorbis" {set Files [glob -nocomplain *.{O,o}{G,g}{G,g}]} \
    "Mp3"        {set Files [glob -nocomplain *.{M,m}{P,p}3]} \
    "Flac"       {set Files [glob -nocomplain *.{F,f}{L,l}{A,a}]} \
    "Wav"        {set Files [glob -nocomplain *.{W,w}{A,a}{V,v}]} \
    default      {set Files ""}
  }
  
  if {($Files ne "")} \
  {
    set Files [lsort $Files]
  }
    
  return $Files
}


#---converts a compressed file into a wave file
proc ::tk_cdconverter::LibCompressed2Wav {CompressedFile WavFile Format} \
{
  switch $Format \
  {
    "Ogg-vorbis" {set ReturnValue [::FR_exec::Start "oggdec -o $WavFile $CompressedFile"]} \
    "Mp3"        {set ReturnValue [::FR_exec::Start "mpg321 --wav $WavFile --stereo $CompressedFile"]} \
    "Flac"       {set ReturnValue [::FR_exec::Start "flac --decode -o $WavFile $CompressedFile"]} \
    "Wav"        {set ReturnValue [::FR_exec::Start "cp $CompressedFile $WavFile"]} \
    default      {set ReturnValue [list 1 "::tk_cdconverter::LibCompressed2Wav: wrong format: $Format"]} \
  }

  if {[lindex $ReturnValue 0] != 0} \
  {
    # was anything wrong we give back the error reason
    return [lindex $ReturnValue 1]
  }
  
  return 0
}


#---converts a wave file into a compressed file
proc ::tk_cdconverter::LibWav2Compressed {WavFile CompressedFile Format {Bitrate 0}} \
{
  switch $Format \
  {
    "Ogg-vorbis" {set ReturnValue [::FR_exec::Start "oggenc -b $Bitrate -o $CompressedFile $WavFile"]} \
    "Mp3"        {set ReturnValue [::FR_exec::Start "lame -b $Bitrate -m s $WavFile $CompressedFile"]} \
    "Flac"       {set ReturnValue [::FR_exec::Start "flac -o $CompressedFile $WavFile"]} \
    "Wav"        {set ReturnValue [::FR_exec::Start "cp $WavFile $CompressedFile"]} \
    default      {set ReturnValue [list 1 "::tk_cdconverter::LibWav2Compressed: wrong format: $Format"]} \
  }

  if {[lindex $ReturnValue 0] != 0} \
  {
    # was anything wrong we give back the error reason
    return [lindex $ReturnValue 1]
  }
  
  return 0
}


#---generates the file name for the output file
proc ::tk_cdconverter::LibOutputFileName {File Directory Format} \
{
  set Name [file join $Directory [file rootname $File]]
  
  switch $Format \
  {
    "Ogg-vorbis" {append Name ".ogg"} \
    "Mp3"        {append Name ".mp3"} \
    "Flac"       {append Name ".fla"} \
    "Wav"        {append Name ".wav"} \
    default      {set Name ""} \
  }
  
  return $Name
}


#---generates the file name
proc ::tk_cdconverter::LibCreateFileName {Name Index Format} \
{
  append Name [format "%02d" $Index]
    
  switch $Format \
  {
    "Ogg-vorbis" {append Name ".ogg"} \
    "Mp3"        {append Name ".mp3"} \
    "Flac"       {append Name ".fla"} \
    "Wav"        {append Name ".wav"} \
    default      {set Name ""} \
  }
  
  return $Name
}


#----if the outputfile already exists - we ask for deleting
proc ::tk_cdconverter::LibCheckOutputFile {OutputFile} \
{
 variable VAR
  
 if {![file exists $OutputFile] || $VAR(DeleteAllOutputfiles)} \
 {
   return 1
 }
    
 # the toplevel
 set Window [::FR_emptyBox::Create -window .check_outputfile -title [msgcat::mc "The output file already exists"]]

 # the frames
 frame .$Window.textframe -relief flat
 frame .$Window.buttonframe -relief flat
 
 set TextFrame "$Window.textframe"
 set ButtonFrame "$Window.buttonframe"
 
 pack .$TextFrame -side top -expand yes -fill both -padx 5 -pady 5
 pack .$ButtonFrame -side top -fill x -padx 5 -pady 5
 
 # the text widget  
 set TextWidget [::FR_general::TextwidgetWithScrollbars .$TextFrame]
    
 .$TextWidget insert end [msgcat::mc "The file \"%s\" allready exists.\n\nShould we delete it?" $OutputFile]
 .$TextWidget insert end "\n\n"
    
 checkbutton .$TextWidget.cb1 -text [msgcat::mc "Delete all output files with a name-conflict in this directory"] -background white -activebackground white -highlightbackground white -variable ::tk_cdconverter::VAR(DeleteAllOutputfiles)
 
 .$TextWidget window create end -window .$TextWidget.cb1
 
 # the bottom buttons
 set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Yes"] [msgcat::mc "Cancel"]]]
 
 button .$ButtonFrame.okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::tk_cdconverter::LibCheckOutputFileAction $Window 1]
 button .$ButtonFrame.cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::tk_cdconverter::LibCheckOutputFileAction $Window 0]
        
 pack .$ButtonFrame.okay -side left -padx {5 0} -pady 5 -expand yes -fill x
 pack .$ButtonFrame.cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
 bind .$Window <KeyPress-Return> [list .$ButtonFrame.okay invoke]
 bind .$Window <KeyPress-Escape> [list .$ButtonFrame.cancel invoke]
  
 ::FR_emptyBox::Show -window .$Window -returnvalue 0

 vwait ::tk_cdconverter::VAR(LibCheckOutputFileReturnValue)
 
 return $VAR(LibCheckOutputFileReturnValue)
}


proc ::tk_cdconverter::LibCheckOutputFileAction {Window Action} \
{
  variable VAR
  
  # in case of a cancel command we set VAR(DeleteAllOutputfiles) back
  if {$Action == 0} \
  {
    set VAR(DeleteAllOutputfiles) 0 
  }
  
  # Action is equal to the returnvalue
  set VAR(LibCheckOutputFileReturnValue) $Action
  
  focus .
  destroy .$Window
}


#---set the state of the bitrate-button depending on the chose outputfile format
proc ::tk_cdconverter::LibBitrateButtonState {ComboBox} \
{
  variable CONFIG
  
  switch $CONFIG(OutputFormat) \
  {
    "Ogg-vorbis" {::FR_comboBox::cb .$ComboBox -state normal} \
    "Mp3"        {::FR_comboBox::cb .$ComboBox -state normal} \
    default      {::FR_comboBox::cb .$ComboBox -state disable}
  }
}